function [idx] = gene2idx(myconn, ref, genelist, type)
% [idx] = gene2idx(conn, ref, genelist, type)
% e.g. idx = gene2idx(conn, emesh, genelist, 'file_fbgn_idx');

% Speedup: create table with emesh indices
%
% for i=1:length(emesh.files),
% sql_fid = pg_query(conn, 1, 'select file_id from file where name = ''%s''', emesh.files{i});
% pg_query(conn, 0, 'insert into file2emesh(file, emesh_idx) values(%d, %d)', sql_fid, i);
% end

    verbose = 2;
    
    got_idx = [];
    gl_entries = unique(genelist);
    
    switch(type),
%         case 'file_fbgn'
%             st_sel = 'select f.name from file f, pm_image_quick_info qi where qi.file = f.file_id and qi.fbgn = ''%s''';
%         case 'file_fbgn_idx'
%             st_sel = 'select e.emesh_idx from file2emesh e, file f, pm_image_quick_info qi where qi.file = f.file_id and f.file_id = e.file and qi.fbgn = ''%s''';
%             got_idx = 1;
        case 'sym_fbgn'
            st_sel = 'select distinct ltrim(flybase_name) from main where flybase_id = ''%s''';
        case 'sym_cg'
            st_sel = 'select distinct ltrim(flybase_name) from main where cgname = ''%s''';
        otherwise
            error('Type %s not recognized', int_type);
    end
    
    
    if verbose > 0, fprintf('Found %d unique entries\n', length(gl_entries)); end
                
    idx = [];          
    for i=1:length(gl_entries),
        if isempty(got_idx),
            fn = pg_query(myconn, 0, st_sel, gl_entries{i});
            if verbose > 1, fprintf('%s: %d entries', gl_entries{i}, length(fn)); end
            if isempty(fn),
                continue;
            end
            e_idx = [];
            for j=1:length(fn),
                if isempty(fn{j})
                    fprintf(' empty match');
                    continue;
                end
                if strmatch('null', fn{j}) == 1,
                    fprintf(' null match');
                    continue;
                end
                    
                c_idx = strmatch(fn{j}, ref, 'exact');
                if ~isempty(c_idx), 
                    e_idx = [e_idx, c_idx];
                end
            end
            if verbose > 1, fprintf(', %d references\n', length(e_idx)); end
            idx = [idx e_idx];
        else
            e_idx = pg_query(myconn, 1, st_sel, gl_entries{i});
            % Check for 'No data', returned as cell
            if ~iscell(e_idx)
                idx = [idx e_idx'];
                if verbose > 1, fprintf('%s: %d meshes\n', gl_entries{i}, length(e_idx)); end
            else
                if verbose > 1, fprintf('%s: no meshes\n', gl_entries{i}); end
            end
        end
    end 

end