function [eg_res] = manual_label(emesh, m_ref, varargin)
% disp_moverlay(emesh, m_ref, m_res, gmm_stain, conn)
% displays composites of reference (m_ref) with others (m_res)
% emesh is standard emesh struct
% m_ref and m_res are indices for emesh
% m_ref must have size of 1, m_res is an array
% m_ref can be also logical, then a synthetic mesh is constructed
% m_res contains the results that are displayed overlayed
%       can be either an index to emesh or a struct
%       if it's a struct, idx contains the indices and stain the meshes.
%       The meshes in emesh will be replaced by the meshes in m_res.
% gmm_stain can be either a mixture model for each triangle or the
%       precomputed GMM probabilities for each m_res
% conn is optional, if given will query the gene symbols


   % Initialize variables
   counter = 1;
   stainshow = 'Staining';
   showref = 0;
   conn = [];
   selt = 0;
   dump_binary = 0;
   m_res = [];
   
   
    if ~isempty(varargin)
        
        i = 1;
        while i <= length(varargin),
            switch(varargin{i}),
                case 'db'
                    i = i+1;
                    conn = varargin{i};
                case 'binary'
                    i = i+1;
                    m_res = varargin{i};
                otherwise
                    error('Argument %s not known', varargin{i});
            end
            i = i+1;
        end
    end

  
   if isempty(m_res),
       t_conn = fully_connected(emesh);
       m_res = tibin(t_conn, m_ref);
   end
   
   tristain = (1 - m_res) .* 255;
      
   triannot{1} = 'None selected';
   triannot{2} = 'Triangle %d';
      
   %  Create and then hide the GUI as it is being constructed.
   % minimum [360,500,600,300] w/o list
   f = figure('Visible','off','Position', [360,500,700,430], ...
       'Units', 'pixels', 'WindowButtonDownFcn', {@mousebutton_Callback});
 
   %  Construct the components
   htri = uicontrol('Style','text','String', triannot{1},...
          'Position',[500, 260, 150,30]);
   
   hnone = uicontrol('Style','pushbutton','String','None',...
          'Position',[500,200,70,25],...
          'Callback',{@button_none_Callback});
   hall = uicontrol('Style','pushbutton','String','All',...
          'Position',[500,170,70,25],...
          'Callback',{@button_all_Callback});
   hreset = uicontrol('Style','pushbutton','String','Reset',...
          'Position',[500,140,70,25],...
          'Callback',{@button_reset_Callback});
   hdump = uicontrol('Style','pushbutton','String','Dump',...
          'Position',[500,20,70,25],...
          'Callback',{@button_dump_Callback});
      
   hsref = uicontrol('Style', 'checkbox', 'String', 'Dump as binary',...
          'Min', 0, 'Max', 1, 'Value', 0, 'Position',[500, 50, 150,30],...
          'Callback', {@showref_Callback});
            
   hstain = uibuttongroup('visible','off', 'Units', 'pixels', 'Position',[500 300 150 110]);
   hs_u1 = uicontrol('Style','Radio','String','Staining',...
          'pos',[10 70 100 30],'parent',hstain,'HandleVisibility','off');
   hs_u2 = uicontrol('Style','Radio','String','Overlay',...
          'pos',[10 40 100 30],'parent',hstain,'HandleVisibility','off');
   set(hstain,'SelectionChangeFcn',@selstain_Callback);
   set(hstain,'SelectedObject', hs_u1); 
   set(hstain,'Visible','on');      
   
   hp = uipanel('Units','Pixels','Position',[50,10,400,400]); 
   h_aref = axes('Parent', hp, 'Units','Pixels','Position',[20,20,340,160]);
   h_aov = axes('Parent', hp, 'Units','Pixels','Position',[20,220,340,160]);
%   align([hforw,hback,htext],'Center','None');
      
   % Initialize the GUI.
   %Change units to normalized so components resize automatically.
   set([f,h_aref, h_aov, htri, hnone, hall, hreset, hdump, hsref, hstain, hs_u1, hs_u2],...
   'Units','normalized');
   %Create a plot in the axes.
   displaypanels({'binary', 'reference'}, 0);
   % Assign the GUI a name to appear in the window title.
   set(f,'Name','Manual labeling')
   % Move the GUI to the center of the screen.
   movegui(f,'center')
   % Make the GUI visible.
   set(f,'Visible','on');
 
   
   %  Callbacks for simple_gui. These callbacks automatically have 
   %  access to component handles and initialized data because
   %  they are nested at a lower level.
   
   
    function button_none_Callback(source,eventdata)
        tristain(:) = 255;
        displaypanels({'binary', 'reference'}, 0);
    end

    function button_all_Callback(source,eventdata)
        tristain(:) = 0;
        displaypanels({'binary', 'reference'}, 0);
    end

    function button_reset_Callback(source,eventdata)
        tristain = (1 - m_res) .* 255;
        displaypanels({'binary', 'reference'}, 0);
    end
    function button_dump_Callback(source,eventdata)
        if dump_binary == 1,
            outp = tristain == 0;
        else
            outp = find(tristain == 0);
        end
        fprintf('[');
        for i=1:length(outp) - 1,
            fprintf('%d,',outp(i));
        end
        fprintf('%d];\n',outp(length(outp)));
    end

    function showref_Callback(source, eventdata)
        dump_binary = get(source,'Value');
    end

    function selstain_Callback(source, eventdata)
%        disp(source);
%        disp([eventdata.EventName,'  ',... 
%             get(eventdata.OldValue,'String'),'  ', ...
%             get(eventdata.NewValue,'String')]);
%        disp(get(get(source,'SelectedObject'),'String'));
         % Determine the selected data set.
         stainshow = get(get(source,'SelectedObject'),'String');
         displaypanels({'binary', 'reference'}, 0);
    end

    function mousebutton_Callback(source, eventdata)
        save_units = get(f, 'Units');
        set(f, 'Units', 'pixels');
        %figpos = get(f, 'Position');
        point = get(f,'CurrentPoint');
        set(source, 'Units', save_units);
        %x = point(1);
        %y = point(2);

        save_units = get(h_aov, 'Units');
        set(h_aov, 'Units', 'pixels');
        axespos = get(h_aov, 'Position');
        set(h_aov, 'Units', save_units);
        
        save_units = get(hp, 'Units');
        set(hp, 'Units', 'pixels');
        panelpos = get(hp, 'Position');
        set(hp, 'Units', save_units);

        t = emesh.t;
        p_scale = emesh.p_scale;
        
        arect = [ axespos(1) + panelpos(1), axespos(2) + panelpos(2), axespos(3), axespos(4) ];
        if rectint(arect, [point 1 1]) == 0
            return;
        end
        
        prect = [min(p_scale), max(p_scale) - min(p_scale)];
        x = (point(1) - arect(1)) * prect(3) / arect(3) + prect(1);
        y = (arect(2) + arect(4) - point(2)) * prect(4) / arect(4) + prect(2);
                
        x1 = p_scale(t(:,1),1); y1 = p_scale(t(:,1),2);
        x2 = p_scale(t(:,2),1); y2 = p_scale(t(:,2),2);
        x3 = p_scale(t(:,3),1); y3 = p_scale(t(:,3),2);
        x0 = repmat(x, [length(t),1]); y0 = repmat(y, [length(t),1]);

        b0 =  (x2 - x1) .* (y3 - y1) - (x3 - x1) .* (y2 - y1);
        b123 = [ ((x2 - x0) .* (y3 - y0) - (x3 - x0) .* (y2 - y0)) ./ b0, ...
        ((x3 - x0) .* (y1 - y0) - (x1 - x0) .* (y3 - y0)) ./ b0, ...
        ((x1 - x0) .* (y2 - y0) - (x2 - x0) .* (y1 - y0)) ./ b0 ];

        number = find(min(b123') > 0);
        if number > 0,
            selt = number;

            annot = sprintf(triannot{2}, number);
            set(htri, 'String', annot);
            if tristain(number) == 0, 
                tristain(number) = 255;
            else
                tristain(number) = 0;
            end
            displaypanels({'binary', 'reference'}, selt); 
            
            selt = 0;
        end
        
        
    end

    function [selected] = selectimages(number)
        selected = find(eg_res(number,:) == 0);
        if isempty(selected),
            questdlg('No genes with expression. Showing full list.', ' OK ');
            selected = 1:length(m_res);
        end
        
        % update the listbox
        for i=1:length(selected),
            selfiles{i} = m_resfiles{selected(i)};
        end
        counter = 1;
        set(hlist, 'String', selfiles);
        set(hlist, 'Value', m_selr(counter));
        set(htext, 'String', m_resfiles{m_selr(counter)});
    end


    function displaypanels(panels, no)
        
        if strmatch('binary', panels),
            
            axes(h_aov); displaymeshExt(tristain, [], no, emesh);
            
        end
        if strmatch('reference', panels),
            switch stainshow,
                case 'Staining'
                    if no == 0
                        axes(h_aref); displaymeshExt(m_ref, [], 0, emesh);
                    end
                case 'Overlay'
                    axes(h_aref); displaymeshExt(m_ref, tristain, no, emesh);
                    % TODO: overlays
            end            
        end
    end



end
    
    
function displaymeshExt(compmesh, refmesh, highlite, emesh)

    axis ij; axis off;
    hold on;
    
    if highlite > 0,
        if isempty(refmesh),
            cmesh = repmat(compmesh(highlite) ./ 255, [1 3]);
        else
            cmesh = zeros(1, 3);
            cmesh(1) = (255 - refmesh(highlite)) ./ 255;
            cmesh(2) = (255 - compmesh(highlite)) ./ 255;
        end
        roic = fix(emesh.p_scale(emesh.t(highlite,:),1)');
        roir = fix(emesh.p_scale(emesh.t(highlite,:),2)');
        fill(roic,roir,cmesh, 'EdgeColor', cmesh);
    else
        
        if isempty(refmesh),
            cmesh = repmat(compmesh ./ 255, [1 3]);
        else
            cmesh = zeros(length(refmesh), 3);
            cmesh(:,1) = (255 - refmesh) ./ 255;
            cmesh(:,2) = (255 - compmesh) ./ 255;
        end
        
        for i = 1:size(emesh.t,1)
            roic = fix(emesh.p_scale(emesh.t(i,:),1)');
            roir = fix(emesh.p_scale(emesh.t(i,:),2)');
            fill(roic,roir,cmesh(i,:), 'EdgeColor', cmesh(i,:));
        end;

    end
        
end


function displaytri(no, mesh, emesh)
   
    cmesh = repmat(mesh(no) ./ 255, [1 3]);
    roic = fix(emesh.p_scale(emesh.t(no,:),1)');
    roir = fix(emesh.p_scale(emesh.t(no,:),2)');
    fill(roic,roir,cmesh(no,:), 'EdgeColor', cmesh(no,:));

end


% find the binarized pattern
function [bin] = tibin(t_conn, stain)

    j_e_mrf = exist('JTImrf2');
    if j_e_mrf == 8,
    	jmrf = JTImrf2(t_conn);
    else
        error('Java MRF not found - check your classpath!\n');
    end

    fprintf('Calculating binarized patterns... ');

    % datapoints
    dp = size(stain,1);
    dlen = size(stain,2);
        
    bpat = zeros(dp, dlen);
    for i=1:dlen, 
        jmrf.oneMesh(2, stain(:,i), 2); 
        bpat(:,i) = jmrf.getLabels(1); 
    end

    bin = zeros(dp, dlen);
    for i=1:dlen, 
        if mean(stain(bpat(:,i) == 1,i)) < mean(stain(bpat(:,i) == 2,i)),
            bin(bpat(:,i) == 1,i) = 1;
        else
            bin(bpat(:,i) == 2,i) = 1;
        end
    end

    fprintf('done\n');

end


% from find_restricted
function [t_conn] = fully_connected(emesh)

    for i=1:length(emesh.t)
        tnfr{i} = [];   
        tnf{i} = []; 
        for j=1:size(emesh.t,2)       
            [prow, pcol] = find(emesh.t==emesh.t(i,j));
            tnfr{i} = [tnfr{i}, prow(prow ~= i)'];
        end
        for j=1:length(emesh.tn{i})
           if length(find(tnfr{i} == emesh.tn{i}(j))) > 1,
            tnf{i} = [tnf{i}, emesh.tn{i}(j)];
          end
        end
    end

    t_conn=zeros(311,311);
    for i=1:length(tnf), 
        for j=1:length(tnf{i}), 
            t_conn(i,tnf{i}(j)) = 1; 
            t_conn(i,i) = 1; 
        end;
    end

    t_conn = logical(t_conn);
end

% from displaymesh
function [adj] = adj_mesh(toadj, rmin, rmax)

    if min(toadj) == max(toadj),
        adj = repmat(255, [length(toadj), 1]);
        return;
    end

    to_r = range(toadj);
    to_min = min(toadj);
    adj_r = abs(rmax - rmin);
    
    adj = abs(rmin - ((toadj - to_min) .* (adj_r / to_r)));
    adj(adj > 255) = 255;
end
