import java.lang.StrictMath.*;
import java.util.*;

/*
 * This is essentially a clean up of the meshMRF class (m = modified)
 * Base class for any modifications
*/

public class JTImrf2 {
	
	protected connVect t_conn, t_conn_in, t_conn_s, t_conn_e;
	protected static int tri;
	protected int [] stain;
	protected int [] tl;
	
	// The following variables are public so they can be set/queried directly from Matlab
	public int [] sk;
	public double [] sc;
	// Default paramaters
	public int P_EDGE = 10;
	public double P_EDGE_PENALTY = 0.1;
	public double P_WEIGHT_DATA = 5;
	public double P_WEIGHT_LABELS = 1;
	public int MAX_ITERATION = 200;
	public boolean invert = false;
	// Max/min parameter values for parameter training
	public int MAX_EDGE = 200;
	public double MAX_EDGE_PENALTY = 100;
	public double MAX_WEIGHT_DATA = 100;
	public double MAX_WEIGHT_LABELS = 100;
	public int MIN_EDGE = 1;
	public double MIN_EDGE_PENALTY = 0.1;
	public double MIN_WEIGHT_DATA = 0.1;
	public double MIN_WEIGHT_LABELS = 0.1;
	
		
	
	/*
	 * Initialize class with connectivity matrix
	*/
	public JTImrf2(int [][]in_t_conn)
	{
		connMat(in_t_conn);
	}
	
	
	/*
	* This nested class handles the connectivity between triangles
	*/
	
	public class connVect {
		
		// conn = all connections
		// conn_1 = connections in embryo 1
		// conn_2 = connections in embryo 2
		int [][] conn, conn_1, conn_2;
		
		public connVect(int [][] in_conn)
		{
			int i, j;
			Vector<Integer> pos;
			
			conn = new int[in_conn.length][];
			
			for (i=0; i<in_conn.length; i++) {
				
				pos = new Vector<Integer>(10);
				
				for (j=0; j<in_conn[i].length; j++) {
					if (in_conn[i][j] == 1) {
						pos.addElement(new Integer(j));
					}
				}
				
				conn[i] = new int[pos.size()];
				
				for ( j=0; j<pos.size(); j++ ) {
					conn[i][j] = ((Integer)pos.elementAt(j)).intValue();
				}
			}
			
			conn_1 = null;
			conn_2 = null;
		}
		

		public connVect(int [][] in_conn1, int [][] in_conn2, int [][] in_conn3, int [][] in_conn4)
		{
			int i, j;
			tri = in_conn1.length;
			int vec_size;
			Vector<Integer> pos;
			
			conn = new int[tri * 2][];
			
			for (i=0; i<tri; i++) {
				
				pos = new Vector<Integer>(20);
				
				for (j=0; j<tri; j++) {
					if (in_conn1[i][j] == 1) {
						pos.addElement(new Integer(j));
					}
				}
				for (j=0; j<tri; j++) {
					if (in_conn2[i][j] == 1) {
						pos.addElement(new Integer(j+tri));
					}
				}
				
				vec_size = pos.size();
				conn[i] = new int[vec_size];
				for ( j=0; j<vec_size; j++ ) {
					conn[i][j] = ((Integer)pos.elementAt(j)).intValue();
				}


				for (j=0; j<tri; j++) {
					if (in_conn3[i][j] == 1) {
						pos.addElement(new Integer(j));
					}
				}
				for (j=0; j<tri; j++) {
					if (in_conn4[i][j] == 1) {
						pos.addElement(new Integer(j+tri));
					}
				}
				
				conn[i+tri] = new int[pos.size() - vec_size];
				for ( j=vec_size; j<pos.size(); j++ ) {
					conn[i+tri][j-vec_size] = ((Integer)pos.elementAt(j)).intValue();
				}
			}
			
			// Assign split connections
			conn_1 = new int[tri][];
			conn_2 = new int[tri][];
			
			for (i=0; i<tri; i++) {
				int ct_1;
				
				for (ct_1=0; conn[i][ct_1] < tri; ct_1++ );
				conn_1[i] = new int[ct_1+1];
				conn_2[i] = new int[conn[i].length - ct_1];
				
				for (j=0; j < conn[i].length; j++ ) {
					if ( conn[i][j] < tri ) {
						conn_1[i][j] = conn[i][j];
					} else {
						conn_2[i][j-ct_1] = conn[i][j];
					}
				}
				
			}
			
		}

		
		public final int [] connections(int pos)
		{
			int i;
			int [] ret = new int[conn[pos].length];
			
			for ( i=0; i<conn[pos].length; i++ ) {
				ret[i] = conn[pos][i];
			}
			
			return ret;
		}

		public final int [] connections(int pos, int embryo)
		{
			int adj_pos = pos;
			
			if ( pos >= tri ) {
				adj_pos -= tri;
			}
			
			if ( embryo == 1 ) {
				return conn_1[adj_pos];
			}
			else if ( embryo == 2 ) {
				return conn_2[adj_pos];
			}
			else {
				return null;
			}
		}
		
		
		public final int [] posArrayElements(int pos, int [] vals)
		{
			int i;
			int [] ret = new int[conn[pos].length];
			
			for ( i=0; i<conn[pos].length; i++ ) {
				ret[i] = vals[conn[pos][i]];
			}
			
			return ret;
		}

		public final double [] posArrayValues(int pos, int [] refvals, double []  vals)
		{
			int i;
			double [] ret = new double[conn[pos].length];
			
			for ( i=0; i<conn[pos].length; i++ ) {
				ret[i] = vals[refvals[conn[pos][i]]];
			}
			
			return ret;
		}

		public final double [] valuesIntDoubleDebug(int pos, int [] refvals, double []  vals)
		{
			int i;
			double [] ret = new double[conn[pos].length];
			
			System.out.println("pos = "+pos);
			for ( i=0; i<conn[pos].length; i++ ) {
				ret[i] = vals[refvals[conn[pos][i]]];
				System.out.println("i = "+i+", conn= "+ conn[pos][i] + ", ref=" + refvals[conn[pos][i]] + "->ret=" + ret[i]);
			}
			
			return ret;
		}

		public final double [] valuesDouble(int pos, double [] vals)
		{
			int i;
			double [] ret = new double[conn[pos].length];
			
			for ( i=0; i<conn[pos].length; i++ ) {
				ret[i] = vals[conn[pos][i]];
			}
			
			return ret;
		}
	}
	
	
	
	public void connMat(int [][]in_t_conn)
	{
		int i, j;
		int conn_size = in_t_conn.length;
		
		int [][]conn = new int[conn_size][conn_size];
		
		for ( i=0; i<conn_size; i++ ) {
			for ( j=0; j<conn_size; j++ ) {
				if ( i == j ) {
					conn[i][j] = 0;
				} else {
					conn[i][j] = in_t_conn[i][j];
				}
			}
		}
		
		t_conn = new connVect(conn);
		t_conn_in = new connVect(in_t_conn);
		t_conn_e = new connVect(conn, conn, conn, conn);
		t_conn_s = new connVect(conn, in_t_conn, in_t_conn, conn);
		
	}
	
	// Main calling function - returns the energy for single TI
	public final double oneMesh(int no_clus, int [] in_stain, int verbose)
	{
		int i, j;
		double T = 1;
		int tri_change;
		int new_lab;
		int iter = 1;
		double h;
		
		int max_iter = MAX_ITERATION;
		
		stain = new int[tri];
		double en[] = new double[tri];
		sk = new int[tri];
		tl = new int[tri];
		sc = new double[ no_clus ];
		
		Random rand = new Random();
		
		initParameters();
		
		// unify stain arrays and 
		// initial assignments of sk to random values
		for (i=0; i < tri; i++ ) {
			stain[i] = (int) in_stain[i];
			
			sk[i] = rand.nextInt(no_clus);
		}
		
		// calculate the initial energy
		for (i=0; i < tri; i++ ) {
			en[i] = g_energy1(i, sk[i], 0);
		}
		
		// Gibbs sampler loop - iterate until converged with simulated annealing
		// T is the current "temperature"
		while (true) {
			
			tri_change = 0;
			
			updateParameters();
			
			for ( i=0; i < tri; i++ ) {
				
				new_lab = rand.nextInt(no_clus);
				h = g_energy1(i, new_lab, 0);
				
				if ( h < en[i] ) {
					sk[i] = new_lab;
					en[i] = h;
					tri_change = tri_change + 1;
				}
				else if (rand.nextDouble() > 1-T) {
					sk[i] = new_lab;
					en[i] = h;
					tri_change = tri_change + 1;
				}
			}
			
			T = T / StrictMath.log1p((double) max_iter+iter) * StrictMath.log1p((double) max_iter);
			
			if ( verbose > 2 ) {
				System.out.println("Iteration " + iter + " -> " + tri_change + " changes");
			}
			
			iter = iter + 1;
			
			if ( tri_change == 0 || iter > max_iter ) {
				break;
			}
		}
		
		h=0;
		for (i=0; i < tri; i++ ) {
			h += en[i];
		}
		return h;
	}

	// Main calling function - returns the energy for the two TIs
	public final double trainParams(int no_clus, int [] in_stain, int [] in_sk, int verbose)
	{
		int i, j;
		double T = 1;
		int param_change, param_no = 0;
		int param_save_i = 0;
		double param_save_d = 0;
		int new_lab;
		int iter = 1;
		double h, h_best;
		
		int max_iter = MAX_ITERATION;
		
		stain = new int[tri];
		sk = new int[tri];
		tl = new int[tri];
		sc = new double[ no_clus ];
		
		Random rand = new Random();
		
		for (i=0; i < tri; i++ ) {
			stain[i] = (int) in_stain[i];
			sk[i] = in_sk[i];
		}
		
		
		// calculate the initial energy
		updateParameters();
		h = 0;
		for (i=0; i < tri; i++ ) {
			h += g_energy1(i, sk[i], 0);
		}
		h_best = h;
		
		// Gibbs sampler loop - iterate until converged with simulated annealing
		// T is the current "temperature"
		while (true) {
			
			param_change = 0;
			
			for ( param_no = 0; param_no < 4; param_no++ ) {
				
				switch (param_no) {
					case 0:
						param_save_i = P_EDGE;
						P_EDGE = rand.nextInt(MAX_EDGE);
						if ( P_EDGE < MIN_EDGE ) { P_EDGE = MIN_EDGE; }
						break;
					case 1:
						param_save_d = P_EDGE_PENALTY;
						P_EDGE_PENALTY = rand.nextDouble() * MAX_EDGE_PENALTY;
						if ( P_EDGE_PENALTY < MIN_EDGE_PENALTY ) { P_EDGE_PENALTY = MIN_EDGE_PENALTY; }
						break;
					case 2:
						param_save_d = P_WEIGHT_DATA;
						P_WEIGHT_DATA = rand.nextDouble() * MAX_WEIGHT_DATA;
						if ( P_WEIGHT_DATA < MIN_WEIGHT_DATA ) { P_WEIGHT_DATA = MIN_WEIGHT_DATA; }
						break;
					case 3:
						param_save_d = P_WEIGHT_LABELS;
						P_WEIGHT_LABELS = rand.nextDouble() * MAX_WEIGHT_LABELS;
						if ( P_WEIGHT_LABELS < MIN_WEIGHT_LABELS ) { P_WEIGHT_LABELS = MIN_WEIGHT_LABELS; }
						break;
				}
				
				h = 0;
				for (i=0; i < tri; i++ ) {
					h += g_energy1(i, sk[i], 0);
				}
				
				if ( h < h_best ) {
					param_change += 1;
					h_best = h;
				} else if ( rand.nextDouble() > 1-T ) {
					param_change += 1;
					h_best = h;
				} else {
					switch (param_no) {
						case 0:
							P_EDGE = param_save_i;
							break;
						case 1:
							P_EDGE_PENALTY = param_save_d;
							break;
						case 2:
							P_WEIGHT_DATA = param_save_d;
							break;
						case 3:
							P_WEIGHT_LABELS = param_save_d;
							break;
					}
				}
				
			}
			
			T = T / StrictMath.log1p((double) max_iter+iter) * StrictMath.log1p((double) max_iter);
			
			if ( verbose > 2 ) {
				System.out.println("Iteration " + iter + " -> " + param_change + " changes");
			}
			
			iter = iter + 1;
			
			if ( param_change == 0 || iter > max_iter ) {
				break;
			}
		}
		
		return h;
	}

	
	
	// Main calling function - returns the energy for the two TIs
	public final double twoMesh(int no_clus, int [] stain1, int [] stain2, boolean boundary, int verbose)
	{
		int i, j;
		double T = 1;
		int tri_change;
		int new_lab;
		int iter = 1;
		double h;
		
		int max_iter = MAX_ITERATION;

		stain = new int[2 * tri];
		double en[] = new double[tri];
		sk = new int[tri * 2];
		tl = new int[tri * 2];
		sc = new double[ no_clus ];
		
		Random rand = new Random();
		
		initParameters();
		
		// unify stain arrays and 
		// initial assignments of sk to random values
		for (i=0; i < tri; i++ ) {
			stain[i] = (int) stain1[i];
			stain[i+tri] = (int) stain2[i];
			
			sk[i] = rand.nextInt(no_clus);
			sk[i+tri] = sk[i];
		}
		
		// calculate the initial energy
		for (i=0; i < tri; i++ ) {
			en[i] = g_energy2(i, sk[i], boundary, 0);
		}
		
		// Gibbs sampler loop - iterate until converged with simulated annealing
		// T is the current "temperature"
		while (true) {
			
			tri_change = 0;
			
			updateParameters();
			
			for ( i=0; i < tri; i++ ) {
				
				new_lab = rand.nextInt(no_clus);
				h = g_energy2(i, new_lab, boundary, 0);
				
				if ( h < en[i] ) {
					sk[i] = new_lab;
					sk[i+tri] = new_lab;
					en[i] = h;
					tri_change = tri_change + 1;
				}
				else if (rand.nextDouble() > 1-T) {
					sk[i] = new_lab;
					sk[i+tri] = new_lab;
					en[i] = h;
					tri_change = tri_change + 1;
				}
			}
			
			T = T / StrictMath.log1p((double) max_iter+iter) * StrictMath.log1p((double) max_iter);
			
			if ( verbose > 2 ) {
				System.out.println("Iteration " + iter + " -> " + tri_change + " changes");
			}
		
			iter = iter + 1;
			
			if ( tri_change == 0 || iter > max_iter ) {
				break;
			}
		}
		
		h=0;
		for (i=0; i < tri; i++ ) {
			h += en[i];
		}
		return h;
	}
	
	
	// Initialize the parameters before iterating
	protected void initParameters()
	{
		Random rand = new Random();
		
		// initial assignments of sc to random values
		for (int i=0; i < sc.length; i++) {
			sc[i] = rand.nextDouble() * 256;
		}
				
	}
	
	
	protected void updateParameters()
	{
		for ( int i=0; i < sc.length; i++ ) {
			sc[i] = mean(stain, i);
		}
		
	}
	
	// Set anti-correlation to yes (1) or no (0)
	public void inverse(int inv) {
		if ( inv == 0 ) {
			invert = false;
		} else { 
			invert = true;
		}
	}
	

	// Returns the current anti-correlation
	public int inverse()
	{
		if (invert) {
			return 1;
		} else {
			return 0;
		}
	}
	
	// Calculates the energy around the current triangle for single TI
	protected final double g_energy1(int pos, int nlabel, int debug)
	{
		int i;
		double dc;
		int [] dn;
		double fc; 
		double [] fn;
		double h;
		int olabel = sk[pos];
		int [] sk_temp = sk;
		
		// Instead of copying sk into sk_temp every time, just change
		// the relevant part - must be set back at end of function
		sk_temp[pos] = nlabel;
		
		// find the intensities for triangles in direct vicinty
		dc = stain[pos];
		dn = t_conn.posArrayElements(pos, stain);
		
		// find the labels for triangles in direct vicinty
		fc = sc[nlabel];
		fn = t_conn.posArrayValues(pos, sk_temp, sc);
		
		double [] edge = new double[dn.length];
		for ( i=0; i < dn.length; i++ ) {
			if ( StrictMath.abs(dn[i] - dc) > P_EDGE ) {
				edge[i] = 1;
			}
		}
		
		double d_f = (dc - fc);
		double d_f1;
		d_f = d_f * d_f;
		for ( i=0; i < dn.length; i++) {
			d_f1 = dn[i] - fn[i];
			d_f += d_f1 * d_f1;
		}
		
		double f = 0;
		double f_p_sq;
		for ( i=0; i < fn.length; i++ ) {
			f_p_sq = (fn[i] - fc);
			f += f_p_sq * f_p_sq  * (1-edge[i]) + P_EDGE_PENALTY * edge[i];
		}
		
		h = P_WEIGHT_DATA * d_f + P_WEIGHT_LABELS * f;
		
				
		// Reset sk_temp to original
		sk_temp[pos] = olabel;
		
		return h;
		
	}
	
	
	
	// Calculates the energy around the current triangle for both TIs
	protected final double g_energy2(int pos, int nlabel, boolean boundary, int debug)
	{
		int i;
		int [] cn, cne, b, be;
		double dc;
		int [] dn, dne;
		double fc; 
		double [] fn, fne;
		double h;
		int olabel = sk[pos];
		int [] sk_temp = sk;
		
		// Instead of copying sk into sk_temp every time, just change
		// the relevant part - must be set back at end of function
		sk_temp[pos] = nlabel;
		sk_temp[pos+tri] = nlabel;
		
		// find the intensities for triangles in direct vicinty
		dc = stain[pos];
		dn = t_conn_s.posArrayElements(pos, stain);
		dne = t_conn_e.posArrayElements(pos, stain);
		
		// find the labels for triangles in direct vicinty
		fc = sc[nlabel];
		fn = t_conn_s.posArrayValues(pos, sk_temp, sc);
		fne = t_conn_e.posArrayValues(pos, sk_temp, sc);		
		
		b = new int[dn.length];
		be = new int[dne.length];
		if ( boundary == true ) {
			// score second embryo only if triangle at boundary
			// represented by b/be
			cn = t_conn_s.connections(pos,2);
			cne = t_conn_e.connections(pos,2);
			for ( i = 0; i < dn.length - cn.length; i++ ) {
				b[i] = 1;
			}
			for ( i = dn.length - cn.length; i < dn.length; i++ ) {
				b[i] = border(cn[i - (dn.length - cn.length)]);
				fn[i] = condInvert(fn[i]);
			}
			for ( i = 0; i < dne.length - cne.length; i++ ) {
				be[i] = 1;
			}
			for ( i = dne.length - cne.length; i < dne.length; i++ ) {
				be[i] = border(cne[i - (dne.length - cne.length)]);
				fne[i] = condInvert(fne[i]);
			}
		} else {
			// score second embryo everywhere
			for ( i=0; i < dn.length; i++ ) {
				b[i] = 1;
			}
			for ( i=0; i < dne.length; i++ ) {
				be[i] = 1;
			}
		}
				
		double [] edge = new double[dne.length];
		for ( i=0; i < dne.length; i++ ) {
			if ( StrictMath.abs(dne[i] - dc) * be[i] > P_EDGE ) {
				edge[i] = 1;
			}
		}
		
		double d_f = (dc - fc);
		double d_f1;
		d_f = d_f * d_f;
		for ( i=0; i < dn.length; i++) {
			d_f1 = dn[i] - fn[i];
			d_f += d_f1 * d_f1 * b[i];
		}
		
		double f = 0;
		double f_p_sq;
		for ( i=0; i < fne.length; i++ ) {
			f_p_sq = (fne[i] - fc) * be[i];
			f += f_p_sq * f_p_sq  * (1-edge[i]) + P_EDGE_PENALTY * edge[i];
		}
		
		h = P_WEIGHT_DATA * d_f + P_WEIGHT_LABELS * f;
		
		
		if ( debug > 0 ) {
			
			for (i=0; i<sc.length; i++) {
				System.out.println("sc[" + i + "]=" + sc[i]);
			}
			System.out.println("dc=" + dc + ", fc=" + fc);
			for (i=0; i<dn.length; i++) {
				System.out.print("dn[" + i + "]=" + dn[i] + ", fn[" + i + "]=" + fn[i]);
				System.out.println(", border=" + b[i]);
			}
			for (i=0; i<dne.length; i++) {
				System.out.print("dne[" + i + "]=" + dne[i] + ", fne[" + i + "]=" + fne[i]);
				System.out.println(", edge=" + edge[i] + ", border=" + be[i]);
			}
			
			System.out.println("d_f="+d_f);
			System.out.println("f="+f);
			System.out.println("h="+h);
		}

		// Reset sk_temp to original
		sk_temp[pos] = olabel;
		sk_temp[pos+tri] = olabel;
		
		return h;
		
	}

	
	// This function returns the maximum staining intensity or minimum staining intensity depending on the "invert" value
	protected final double condInvert(double f)
	{
		if ( invert ) {
			double fmin = min(sc);
			if ( f == fmin ) {
				return max(sc);
			} else {
				return fmin;
			}
		}
		
		return f;
	}
	
	// Is the triangle at the current boundary of the detected staining, 1 if yes, 0 if no
	protected final int border(int pos)
	{
		double sc_min = min(sc);
		
		if ( sc[sk[pos]] == sc_min ) {
			return 1;
		}
		
		int [] pos_conn = t_conn_s.connections(pos, 1);
		
		for ( int i = 0; i < pos_conn.length; i++ ) {
			if ( sc[sk[pos_conn[i]]] == sc_min ) {
				return 1;
			}
		}
		
		return 0;

	}


	// Finds the minimum - quick hack, probably a more efficient way to do it
	protected final double min(double [] val)
	{
		if ( val.length < 3 ) {
			return val[0] < val[1] ? val[0] : val[1];
		}
		else {
			double min_val = Double.MAX_VALUE;
			for (int i=0; i<val.length; i++ ) {
				if (val[i] < min_val) {
					min_val = val[i];
				}
			}
			return min_val;			
		}
	}

	// Finds the maximum - quick hack, probably a more efficient way to do it
	protected final double max(double [] val)
	{
		if ( val.length < 3 ) {
			return val[0] > val[1] ? val[0] : val[1];
		}
		else {
			double max_val = Double.MIN_VALUE;
			for (int i=0; i<val.length; i++ ) {
				if (val[i] > max_val) {
					max_val = val[i];
				}
			}
			return max_val;			
		}
	}


	protected final double mean(int []a, int clab)
	{
		int i;
		int start, end;
		double sum, ct;
		
		sum = 0;
		ct = 0;
				
		for (i = 0; i < a.length; i++) {
			if ( sk[i] == clab ) {
				sum += a[i];
				ct += 1;
			}
		}
	
		
		return sum/ct;
	}


	/*
	Queries
	*/
	
	// Returns the labels for all triangles
	public final double [] getLabels(int stain_no)
	{
		int i,j;
		int start, end;
		double [] label;
		
		if ( sk.length == 0 ) {
			System.out.println("No label assignments yet");
			return null;
		}
		
		label = new double[tri];
		
		if ( stain_no == 1 ) {
			start = 0;
			end = tri;
		} else if ( stain_no == 2 ) {
			start = tri;
			end = sk.length;
		} else {
			System.out.println("Non-existent stain");
			return null;
		}
			
		for ( i = start, j=0; i < end; i++, j++ ) {
			label[j] = (double) sk[i]+1;
		}
		
		return label;
	}
	
	// Returns the staining intensity for each label
	public final double [] getCenters()
	{
		return sc;
	}
	
	// Dumps the connectivity matrix for debugging purposes
	public final void dumpConnMat(int choice)
	{
		int i,j;
		
		connVect conn;
		
		switch(choice) {
			case 1: conn=t_conn;
				break;
			case 2: conn=t_conn_e;
				break;
			case 3: conn=t_conn_s;
				break;
			default: conn=t_conn;
				break;
		}
		
		for (i=0; i<tri; i++) {
			int [] c = conn.connections(i);
			System.out.print(i + ": ");
			for (j=0; j<c.length; j++) {
				System.out.print(c[j] + ", ");
			}
			System.out.println();
		}
	}
	
	
	public final void printParams()
	{
		System.out.print("MAX_ITERATION = " + MAX_ITERATION + "\n");
		System.out.print("P_EDGE = " + P_EDGE + "\n");
		System.out.print("P_EDGE_PENALTY = " + P_EDGE_PENALTY + "\n");
		System.out.print("P_WEIGHT_DATA = " + P_WEIGHT_DATA + "\n");
		System.out.print("P_WEIGHT_LABELS = " + P_WEIGHT_LABELS + "\n");
	}	
	
}

